/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: ExclusionRangeForm.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public partial class ExclusionRangeForm : Form
    {
        private SubnetScopeNode subnetNode;
        public ExclusionRangeForm(SubnetScopeNode node)
        {
            InitializeComponent();
            this.subnetNode = node;
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (_AddSubnetExclusion())
            {
                this.DialogResult = DialogResult.OK;
            }
        }

        private Boolean _AddSubnetExclusion()
        {
            DhcpIpRange exRange = new DhcpIpRange();
            exRange.Type = DhcpSubnetElementDataType.DhcpExcludedIpRanges;
            if (!MyValidate(out exRange))
                return false;

            try
            {
                this.subnetNode.ProfileNode.DhcpOps.CreateSubnetExclusion(this.subnetNode.ServerIp,
                    this.subnetNode.DhcpSubnet.Address, exRange);
            }
            catch (Exception e)
            {
                MessageBox.Show(this, "Add Exclusion Failed" + Environment.NewLine +
                      "Error: " + e.Message, "Exclusion Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            return true;
        }

        private Boolean MyValidate(out DhcpIpRange exRange)
        {
            String caption = "Input Error";
            DhcpSubnet network = this.subnetNode.DhcpSubnet;
            //start ip

            exRange = new DhcpIpRange();
            exRange.Type = DhcpSubnetElementDataType.DhcpExcludedIpRanges;

            DhcpIpAddress startIp = null;
            try
            {
                startIp = exRange.StartAddress = new DhcpIpAddress(tbStartIp.Text.Trim());
            }
            catch
            {
                MessageBox.Show(this, "Start IP not valid", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            DhcpIpAddress endIp = null;
            try
            {
                if (tbEndIp.Text.Trim().Length == 0)
                    endIp = exRange.EndAddress = startIp;
                else
                    endIp = exRange.EndAddress = new DhcpIpAddress(tbEndIp.Text.Trim());
            }
            catch
            {
                MessageBox.Show(this, "End IP not valid", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //check mask
            if ((startIp.GetUIntAddress() & network.Mask.GetUIntAddress()) != network.Address.GetUIntAddress()
                || (endIp.GetUIntAddress() & network.Mask.GetUIntAddress()) != network.Address.GetUIntAddress())
            {
                MessageBox.Show(this, "IP Range not within subnet", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //check sip < eip
            if (!(startIp.GetUIntAddress() <= endIp.GetUIntAddress()))
            {
                MessageBox.Show(this, "IP Range not correct", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            //if sip equal network
            if (startIp.GetUIntAddress() == (startIp.GetUIntAddress() & network.Mask.GetUIntAddress()))
            {
                MessageBox.Show(this, "Start IP cannot equal Network Address", caption, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }

            return true;
        }
    }
}
