/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: OptionControl.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    
    public partial class OptionControl : UserControl
    {
        private OptionPropertyPage parentPage;
        private DhcpOptionClassType currentClass = DhcpOptionClassType.Dhcp;
        private DhcpOption currentOption = null;
        private DhcpOptionValue currentValue = null;
        private List<DhcpOptionValue> tempList;
        private Boolean changed = false;
        private Boolean isInit = false;

        private Dictionary<DhcpOptionClassType, uint> changedOptions = new Dictionary<DhcpOptionClassType, uint>();

        public OptionControl()
        {
            InitializeComponent();
        }

        public OptionControl(OptionPropertyPage parentPage)
        {
            InitializeComponent();
            this.parentPage = parentPage;
        }

        public List<DhcpOptionValue> GetData()
        {
            this.RecordChanges();
            return tempList;
        }


        public void SetData()
        {
            tempList = new List<DhcpOptionValue>(parentPage.OptionNode.OptionValues);

            //set classes
            setUserClasses();

            //set option list
            setOptionsList();

            lvOptions.Focus();
            isInit = true;
        }

        private void setOptionsList()
        {
            ServerScopeNode serverNode = parentPage.OptionNode.ServerNode;

            lvOptions.Items.Clear();
            foreach (DhcpOption opt in serverNode.DhcpOptions.Values)
            {
                lvOptions.Items.Add(new ListViewItem(new String[] { String.Format("{0:d3}", opt.OptionId), opt.Name, opt.Comment }));
            }

            enableArrayInput(false);
        }
        
        private void lvOptions_SelectedIndexChanged(object sender, EventArgs e)
        {
            //record option change if any
            this.RecordChanges();

            changed = false;
            //change to select option (from templist)
            ServerScopeNode serverNode = parentPage.OptionNode.ServerNode;

            if (lvOptions.SelectedItems.Count != 0)
            {
                currentOption = serverNode.DhcpOptions[uint.Parse(lvOptions.SelectedItems[0].SubItems[0].Text)];
                if (currentOption.isArray)
                    this.enableArrayInput(true);
                else
                    this.enableArrayInput(false);

                this.setValueType(currentOption.DataType);

                this.setOptionValue(currentOption);
            }

        }

        public void RecordChanges()
        {
            //update templist if previous option was changed
            if (changed)
            {
                
                //add new optionvalue to templist
                if (currentValue == null)
                {
                    currentValue = new DhcpOptionValue();
                    currentValue.ClassType = currentClass;
                    currentValue.OptionScope = parentPage.OptionNode.OptScope;
                    currentValue.OptionId = currentOption.OptionId;
                    //if array and not empty
                    if (currentOption.isArray && lbValues.Items.Count != 0)
                    {
                        try
                        {
                            List<String> strs = new List<string>(lbValues.Items.Count);
                            foreach (String str in lbValues.Items)
                                strs.Add(str);
                            currentValue.OptionData = DhcpOptionData.GenOptionData(currentOption.DataType, strs.ToArray());
                            tempList.Add(currentValue);
                            this.parentPage.Dirty = true;
                        }
                        catch { SnapInUtil.PopUpError("Input Error", "Input Data Bad"); this.Focus(); }

                    }
                    else if (tbValue.Text.Trim().Length != 0) //if tb not empty
                    {
                        try
                        {
                            currentValue.OptionData = DhcpOptionData.GenOptionData(currentOption.DataType, new String[] { tbValue.Text.Trim() });
                            tempList.Add(currentValue);
                            this.parentPage.Dirty = true;
                        }
                        catch { SnapInUtil.PopUpError("Input Error", "Input Data Bad"); this.Focus(); }
                    }
                }
                else //current value exists
                {
                    foreach (DhcpOptionValue value in tempList)
                    {
                        //if current value matches a value in templist
                        if (value.OptionId == currentValue.OptionId && value.ClassType == currentValue.ClassType)
                        {
                            if (currentOption.isArray)
                            {
                                if (lbValues.Items.Count == 0)
                                {
                                    tempList.Remove(value);
                                    this.parentPage.Dirty = true;
                                }
                                else
                                {
                                    try
                                    {
                                        List<String> strs = new List<string>(lbValues.Items.Count);
                                        foreach (String str in lbValues.Items)
                                            strs.Add(str);
                                        value.OptionData = DhcpOptionData.GenOptionData(currentOption.DataType, strs.ToArray());
                                        this.parentPage.Dirty = true;
                                    }
                                    catch
                                    {
                                        SnapInUtil.PopUpError("Input Error", "Input Data Bad");
                                        this.Focus();
                                    }
                                }
                            }
                            else //not array
                            {
                                if (tbValue.Text.Trim().Length == 0)
                                {
                                    tempList.Remove(value);
                                    this.parentPage.Dirty = true;
                                }
                                else
                                {
                                    try
                                    {
                                        value.OptionData = DhcpOptionData.GenOptionData(currentOption.DataType, new String[] { tbValue.Text.Trim() });
                                        this.parentPage.Dirty = true;
                                    }
                                    catch
                                    {
                                        SnapInUtil.PopUpError("Input Error", "Input Data Bad");
                                        this.Focus();
                                    }
                                }
                            }

                            break;
                        }
                    }
                }
            }
        }

        private void setOptionValue(DhcpOption opt)
        {
            tbValue.Clear();
            lbValues.Items.Clear();

            currentValue = null;
            foreach (DhcpOptionValue optValue in tempList)
            {
                //option has been set under current class selection
                if (opt.OptionId == optValue.OptionId && optValue.ClassType == currentClass)
                {
                    if (opt.isArray)
                    {
                        lbValues.Items.AddRange(optValue.OptionData.GetStrings());
                    }
                    else
                    {
                        tbValue.Text = optValue.OptionData.ToString();
                    }
                    currentValue = optValue;
                    break;
                }
            }
        }

        private void setValueType(DhcpOptionDataType type)
        {
            switch (type)
            {
                case DhcpOptionDataType.ByteType:
                    lblValueType.Text = "Byte";
                    break;
                case DhcpOptionDataType.WordType:
                    lblValueType.Text = "WORD";
                    break;
                case DhcpOptionDataType.DWordType:
                    lblValueType.Text = "DWORD";
                    break;
                case DhcpOptionDataType.DWordDWordType:
                    lblValueType.Text = "DWORD DWORD";
                    break;
                case DhcpOptionDataType.StringDataType:
                    lblValueType.Text = "String";
                    break;
                case DhcpOptionDataType.IpAddressType:
                    lblValueType.Text = "IP Address";
                    break;
            }
        }

        private void setUserClasses()
        {
            DataTable list = new DataTable();
            list.Columns.Add(new DataColumn("Display", typeof(string)));
            list.Columns.Add(new DataColumn("ClassType", typeof(DhcpOptionClassType)));

            list.Rows.Add(list.NewRow());
            list.Rows.Add(list.NewRow());
            list.Rows.Add(list.NewRow());

            list.Rows[0][0] = "Default DHCP Class";
            list.Rows[0][1] = DhcpOptionClassType.Dhcp;
            list.Rows[1][0] = "Default BOOTP Class";
            list.Rows[1][1] = DhcpOptionClassType.Bootp;
            list.Rows[2][0] = "Default Routing and Remote Class";
            list.Rows[2][1] = DhcpOptionClassType.Routing;

            ddlUserClass.DataSource = list;
            ddlUserClass.DisplayMember = "Display";
            ddlUserClass.ValueMember = "ClassType";
            ddlUserClass.SelectedIndex = 0;
        }

        private void enableArrayInput(Boolean enable)
        {
            lbValues.Enabled = btnAdd.Enabled = btnDown.Enabled = btnRemove.Enabled = btnUp.Enabled = enable;
            if (enable)
                lbValues.BackColor = Color.White;
            else
                lbValues.BackColor = Color.LightGray;
        }

        private void ddlUserClass_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (isInit)
            {
                currentClass = (DhcpOptionClassType)ddlUserClass.SelectedValue;
                lvOptions_SelectedIndexChanged(null, null);
                //lvOptions.Items[lvOptions.SelectedIndices[0]].Selected = true;
            }

        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (tbValue.Text.Trim().Length != 0)
            {
                lbValues.Items.Add(tbValue.Text.Trim());
                tbValue.Clear();
            }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            if (lbValues.SelectedItems.Count != 0)
            {
                tbValue.Text = (String)lbValues.SelectedItem;
                lbValues.Items.RemoveAt(lbValues.SelectedIndex);
            }
        }

        private void btnUp_Click(object sender, EventArgs e)
        {
            if (lbValues.SelectedItems.Count != 0)
            {
                if(lbValues.SelectedIndex == 0)
                    return;

                String item = (String)lbValues.SelectedItem;
                int index = lbValues.SelectedIndex;
                lbValues.Items.RemoveAt(index);
                lbValues.Items.Insert(index - 1, item);
                lbValues.SelectedIndex = index - 1;
            }
        }

        private void btnDown_Click(object sender, EventArgs e)
        {
            if (lbValues.SelectedItems.Count != 0)
            {
                if (lbValues.SelectedIndex == lbValues.Items.Count - 1)
                    return;

                String item = (String)lbValues.SelectedItem;
                int index = lbValues.SelectedIndex;
                lbValues.Items.RemoveAt(index);
                lbValues.Items.Insert(index + 1, item);
                lbValues.SelectedIndex = index + 1;
            }
        }

        private void ValueChanged(object sender, EventArgs e)
        {
            //make optionvalue change
            if (lvOptions.SelectedItems.Count != 0)
            {
                this.parentPage.Dirty = true;
                changed = true;
            }

        }
    }
}
