/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: ProfileControl.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Unf.Dhcp.ManagementConsole
{
    public partial class ProfileControl : UserControl
    {
        private ProfilePropertyPage parentPage;

        public ProfileControl()
        {
            InitializeComponent();
            this.parentPage = null;
        }

        public ProfileControl(ProfilePropertyPage parent)
        {
            InitializeComponent();
            this.parentPage = parent;
        }

        public void RefreshData(ProfileScopeNode psn)
        {
            if (psn == null || psn.SaveData == null)
                return;

            this.tbProfileName.Text = psn.SaveData.ProfileName;
            this.tbDWSUrl.Text = psn.SaveData.DwsUrl;
            this.tbUserName.Text = psn.SaveData.UserName;
            this.tbDomainName.Text = psn.SaveData.LoginDomain;

            //if control is being used by propertey page
            if(this.parentPage != null)
                this.parentPage.Dirty = false;
        }

        public ProfileSaveData GetData()
        {
            ProfileSaveData psd = new ProfileSaveData();
            psd.ProfileName = this.tbProfileName.Text.Trim();
            psd.DwsUrl = this.tbDWSUrl.Text.Trim();
            psd.UserName = this.tbUserName.Text.Trim();
            psd.LoginDomain = this.tbDomainName.Text.Trim();
            
            //if control is being used by propertey page
            if(this.parentPage != null)
                this.parentPage.Dirty = false;

            return psd;
        }

        private void tbAny_TextChanged(object sender, EventArgs e)
        {
            //if control is being used by propertey page
            if (this.parentPage != null)
                this.parentPage.Dirty = true;
        }

    }
}
