/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: ServerConnectForm.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public partial class ServerConnectForm : Form
    {
        private ProfileScopeNode profileNode;
        public ServerConnectForm(ProfileScopeNode profileNode)
        {
            this.profileNode = profileNode;
            InitializeComponent();
            tbServerIp.Focus();
        }

        private Boolean TestConnection()
        {
            if (!MyValidate())
                return false;

            try
            {
                profileNode.DhcpOps.EnumSubnets(new DhcpIpAddress(tbServerIp.Text.Trim()));
            }
            catch (Exception e)
            {
                MessageBox.Show("Unable to Connect to Server: " 
                    + tbServerIp.Text.Trim() + Environment.NewLine + "Error: " + e.Message, "Connection Error");
                return false;

            }
            return true;
        }

        private Boolean MyValidate()
        {
            DhcpIpAddress ip = null;
            try
            {
                ip = new DhcpIpAddress(tbServerIp.Text.Trim());
            }
            catch
            {
                MessageBox.Show(this, "Start IP not valid", "Input Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            return true;
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (TestConnection())
                this.DialogResult = DialogResult.OK;
        }
    }
}
