/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: PoolScopeNode.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public class PoolScopeNode : ScopeNode
    {
        public SubnetScopeNode SubnetNode;
        public List<DhcpIpRange> Pools;

        public PoolScopeNode(SubnetScopeNode subnetNode)
        {
            this.SubnetNode = subnetNode;
            Pools = new List<DhcpIpRange>(3);

            this.DisplayName = "Address Pools";
            this.ImageIndex = this.SelectedImageIndex = 6;

            //actions
            this.ActionsPaneItems.Add(new Action("New Exclusion Range...", "Add Exclusion to Subnet", -1, "addrange"));

            //set list view
            MmcListViewDescription lvd = new MmcListViewDescription();
            lvd.DisplayName = "Address Pool(s)";
            lvd.ViewType = typeof(PoolListView);
            lvd.Options = MmcListViewOptions.SingleSelect;
            this.ViewDescriptions.Add(lvd);
            this.ViewDescriptions.DefaultIndex = 0;

            //verbs
            this.EnabledStandardVerbs = StandardVerbs.Refresh;

            this.OnRefresh(null);
        }

        protected override void OnAction(Action action, AsyncStatus status)
        {
            switch ((String)action.Tag)
            {
                case "addrange":

                    ExclusionRangeForm rForm = new ExclusionRangeForm(SubnetNode);

                    if (this.SnapIn.Console.ShowDialog(rForm) == DialogResult.OK)
                    {
                        this.OnRefresh(null);
                    }

                    break;
            }
        }

        protected override void OnRefresh(AsyncStatus status)
        {
            DhcpOperations Ops = this.SubnetNode.ProfileNode.DhcpOps;

            try
            {
                this.Pools.Clear();
                DhcpIpRange[] pools = Ops.EnumPools(this.SubnetNode.ServerIp, this.SubnetNode.DhcpSubnet.Address);
                if(pools != null)
                    this.Pools.AddRange(Ops.EnumPools(this.SubnetNode.ServerIp, this.SubnetNode.DhcpSubnet.Address));
                this.RaiseOnChange();
                
            }
            catch (Exception e)
            {
                SnapInUtil.PopUpError("Pool Error",
                                "Unabled Enumerate Pool" + Environment.NewLine
                                + "Error: " + e.Message);
            }
  
        }

        //Based on Platform SDK Example
        #region Event related methods  ------------------------------------------

        /// <summary>
        /// Declare the delegate type for the changed event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public delegate void ChangedDelegate(object sender, EventArgs ev);

        /// <summary>
        /// internal store of delegates needed when using event 'lock' syntax 
        /// </summary>
        private ChangedDelegate changedDelegate;

        /// <summary>
        /// Changed event to notify that the scope node has changed
        /// </summary>
        public event ChangedDelegate Changed
        {
            add
            {
                lock (this)
                    changedDelegate += value;
            }
            remove
            {
                lock (this)
                    changedDelegate -= value;
            }
        }

        /// <summary>
        /// Raises the changed event using the private delegate
        /// </summary>
        private void RaiseOnChange()
        {
            // safely invoke an event
            ChangedDelegate raiseChangedDelegate = changedDelegate;
            if (raiseChangedDelegate != null)
            {
                raiseChangedDelegate(this, new EventArgs());
            }
        }

        #endregion
    }
}
