/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: RezPropertyPage.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public class RezPropertyPage : PropertyPage
    {
        public RezListView rezList;

        public RezPropertyPage(RezListView rezList)
        {
            //keep track of parent node data
            this.rezList = rezList;

            this.Title = "General";

            //assign control
            this.Control = new RezControl(this);
        }

        protected override void OnInitialize()
        {
            base.OnInitialize();

            this.SetData();
        }

        protected override bool OnApply()
        {
            if (this.Dirty)
            {
                //update reservation here
                DhcpReservation rez;
                Boolean ok = ((RezControl)this.Control).GetData(out rez);
                if (!ok)
                    return false;

                SubnetScopeNode subnetNode = ((RezScopeNode)this.rezList.ScopeNode).SubnetNode;
                try
                {
                    subnetNode.ProfileNode.DhcpOps.UpdateReservation(subnetNode.ServerIp,
                        subnetNode.DhcpSubnet.Address, rez);

                    this.rezList.ResultNodes.Remove((RezResultNode)this.ParentSheet.SelectionObject);
                    this.rezList.ResultNodes.Add(new RezResultNode(rez));
                }
                catch (Exception e)
                {
                    SnapInUtil.PopUpError("Reservation Error", "Update Reservation Failed" + Environment.NewLine +
                      "Error: " + e.Message);
                    return false;
                }

            }
            return true;
        }

        protected override bool OnOK()
        {
            return this.OnApply();
        }

        private void SetData()
        {
            ((RezControl)this.Control).SetData(((RezResultNode)this.ParentSheet.SelectionObject).Reservation);
        }
    }
}
