/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: SubnetAdvPropertyPage.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    public class SubnetAdvPropertyPage : PropertyPage
    {
        public SubnetScopeNode subnetNode;


        public SubnetAdvPropertyPage(SubnetScopeNode subnetNode)
        {
            //keep track of parent node data
            this.subnetNode = subnetNode;

            this.Title = "Advanced";

            //assign control
            this.Control = new SubnetAdvControl(this);
        }

        protected override void OnInitialize()
        {
            base.OnInitialize();

            this.SetData();
        }

        protected override bool OnApply()
        {
            if (this.Dirty)
            {
                DhcpSubnetElementDataType type;
                DhcpOptionValue ov;
                Boolean ok = ((SubnetAdvControl)this.Control).GetData(out type, out ov);

                if (!ok)
                {
                    
                    return false;
                }

                try
                {
                    
                    DhcpIpRange current = ((PoolScopeNode)this.subnetNode.Children[0]).Pools[0];

                    if (type != DhcpSubnetElementDataType.DhcpIpRangesDhcpOnly)
                    {
                        BootpIpRange ipr = new BootpIpRange();
                        ipr.Type = type;
                        ipr.StartAddress = current.StartAddress;
                        ipr.EndAddress = current.EndAddress;
                        ipr.BootpAllocated = 0;
                        ipr.MaxBootpAllowed = uint.MaxValue;
                        this.subnetNode.ProfileNode.DhcpOps.UpdateSubnetRange(this.subnetNode.ServerIp,
                            this.subnetNode.DhcpSubnet.Address, ipr);
                    }
                    else
                    {
                        DhcpIpRange ipr = new DhcpIpRange();
                        ipr.Type = type;
                        ipr.StartAddress = current.StartAddress;
                        ipr.EndAddress = current.EndAddress;
                        this.subnetNode.ProfileNode.DhcpOps.UpdateSubnetRange(this.subnetNode.ServerIp,
                            this.subnetNode.DhcpSubnet.Address, ipr);
                    }
                    ((PoolScopeNode)this.subnetNode.Children[0]).Pools[0].Type = type;
                    
                    //set option 51 for bootp
                    this.subnetNode.ProfileNode.DhcpOps.SetOptionValue(this.subnetNode.ServerIp, ov);
                    
                }
                catch (Exception e)
                {
                    SnapInUtil.PopUpError("Subnet Error", "Update Subnet Failed" + Environment.NewLine +
                      "Error: " + e.Message);
                    return false;
                }
                
            }
            return true;
        }

        protected override bool OnOK()
        {
            return this.OnApply();
        }

        private void SetData()
        {
            DhcpOptionValue value51;

            DhcpOptionScope os = new DhcpOptionScope();
            os.type = DhcpOptionScopeType.Subnet;
            os.SubnetIp = this.subnetNode.DhcpSubnet.Address;
            try
            {
                value51 = this.subnetNode.ProfileNode.DhcpOps.GetOptionValue(this.subnetNode.ServerIp, 51, DhcpOptionClassType.Bootp, os);
            }
            catch
            {
                value51 = null;
            }

            if (((PoolScopeNode)this.subnetNode.Children[0]).Pools.Count != 0)
                ((SubnetAdvControl)this.Control).SetData(((PoolScopeNode)this.subnetNode.Children[0]).Pools[0].Type, value51);
        }
    }
}
