/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: PermissionListView.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.ManagementConsole;

using Unf.Dhcp.ManagementConsole.DwsSec;

namespace Unf.Dhcp.ManagementConsole
{
    class PermissionListView : MmcListView
    {

        protected override void OnInitialize(AsyncStatus status)
        {
            base.OnInitialize(status);

            // Create a set of columns for use in the list view
            // Define the default column title
            this.Columns[0].Title = "Account";
            this.Columns[0].SetWidth(300);


            // Add detail column
            this.Columns.Add(new MmcListViewColumn("Role", 150));

            //listen for scope event
            ((PermissionsScopeNode)this.ScopeNode).Changed += new PermissionsScopeNode.ChangedDelegate(OnScopeNodeChange);

            //allow selected node to be deleted
            this.SelectionData.EnabledStandardVerbs = StandardVerbs.Delete;

            Refresh();
        }

        /// <summary>
        /// Cleanup. 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnShutdown(SyncStatus status)
        {
            //Stop listening for scope event
            ((PermissionsScopeNode)this.ScopeNode).Changed -= new PermissionsScopeNode.ChangedDelegate(OnScopeNodeChange);
        }

        protected void Refresh()
        {
            this.ResultNodes.Clear();

            if (((PermissionsScopeNode)this.ScopeNode).AzScope == null)
            {
                DhcpAzGlobalAccount[] accounts = ((PermissionsScopeNode)this.ScopeNode).ProfileNode.DhcpSec.GetGlobalPermissions();
                if (accounts == null)
                    return;
                foreach (DhcpAzGlobalAccount account in accounts)
                {
                    ResultNode r = new ResultNode();

                    r.DisplayName = account.AccountName ?? String.Empty;
                    r.ImageIndex = 1;
                    r.SubItemDisplayNames.Add(account.GlobalRole.ToString());

                    this.ResultNodes.Add(r);
                }
            }
            else  //not global
            {
                DhcpAzAccount[] accounts = ((PermissionsScopeNode)this.ScopeNode).ProfileNode.DhcpSec.GetPermissions(((PermissionsScopeNode)this.ScopeNode).AzScope);
                if (accounts == null)
                    return;
                foreach (DhcpAzAccount account in accounts)
                {
                    ResultNode r = new ResultNode();

                    r.DisplayName = account.AccountName ?? String.Empty;
                    r.ImageIndex = 1;
                    r.SubItemDisplayNames.Add(account.Role.ToString());

                    this.ResultNodes.Add(r);
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="status"></param>
        protected override void OnSelectionChanged(SyncStatus status)
        {
            if (this.SelectedNodes.Count == 0)
            {
                this.SelectionData.Clear();
            }
            else
            {
                this.SelectionData.Update(this.SelectedNodes[0], false, null, null);
                this.SelectionData.ActionsPaneItems.Clear();
            }
        }

        protected override void OnDelete(SyncStatus status)
        {
            if (((PermissionsScopeNode)this.ScopeNode).AzScope == null)
            {
                try
                {
                    DhcpAzGlobalAccount account = new DhcpAzGlobalAccount();
                    account.AccountName = this.SelectedNodes[0].DisplayName;
                    account.GlobalRole = (DhcpAzGlobalRole)Enum.Parse(typeof(DhcpAzGlobalRole), this.SelectedNodes[0].SubItemDisplayNames[0]);
                    ((PermissionsScopeNode)this.ScopeNode).ProfileNode.DhcpSec.RemoveGlobalPermission(account);

                    this.ResultNodes.Remove((ResultNode)this.SelectedNodes[0]);
                }
                catch (Exception e)
                {
                    SnapInUtil.PopUpError("Permission Delete Error",
                                "Unabled to Delete Permission" + Environment.NewLine
                                + "Error: " + e.Message);
                }
            }
            else //not global
            {
                try
                {
                    DhcpAzAccount account = new DhcpAzAccount();
                    account.AccountName = this.SelectedNodes[0].DisplayName;
                    account.Role = (DhcpAzRole)Enum.Parse(typeof(DhcpAzRole), this.SelectedNodes[0].SubItemDisplayNames[0]);
                    ((PermissionsScopeNode)this.ScopeNode).ProfileNode.DhcpSec.RemovePermission(((PermissionsScopeNode)this.ScopeNode).AzScope, account);

                    this.ResultNodes.Remove((ResultNode)this.SelectedNodes[0]);
                }
                catch (Exception e)
                {
                    SnapInUtil.PopUpError("Permission Delete Error",
                                "Unabled to Delete Permission" + Environment.NewLine
                                + "Error: " + e.Message);
                }
            }
        }

        /// <summary>
        /// Handle any change to the scope node. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void OnScopeNodeChange(object sender,EventArgs ev)
        {
            this.Refresh();
        }
    }
}
