/* 
 * DHCP Web Services MMC 3.0 Snap-In Client
 *
 *  File: SearchListView.cs
 *  Namespace: Unf.Dhcp.ManagementConsole
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Microsoft.ManagementConsole;


using Unf.Dhcp.ManagementConsole.DwsOps;

namespace Unf.Dhcp.ManagementConsole
{
    class SearchListView : MmcListView
    {
        protected override void OnInitialize(AsyncStatus status)
        {
            base.OnInitialize(status);

            // Create a set of columns for use in the list view
            // Define the default column title
            this.Columns[0].Title = "Client IP";
            this.Columns[0].SetWidth(125);

            // Add detail column
            this.Columns.Add(new MmcListViewColumn("Name", 100));
            this.Columns.Add(new MmcListViewColumn("Hardware Address", 100));
            this.Columns.Add(new MmcListViewColumn("Lease Expiration", 125));
            //this.Columns.Add(new MmcListViewColumn("Type", 75));
            this.Columns.Add(new MmcListViewColumn("Description", 175));

            //allow selected node to be deleted
            this.SelectionData.EnabledStandardVerbs = StandardVerbs.Delete;

            // add an action to manual trigger an 'error' handling scenario
            this.ActionsPaneItems.Add(new Action("Clear Results", "Switches the view to default", -1, "ClearSearch"));

            SearchForm sform = new SearchForm();

            if (this.SnapIn.Console.ShowDialog(sform) == DialogResult.OK)
            {
                DhcpClientFilter clf = sform.GetData();

                try
                {
                    DhcpClient[] clients = null;
                    if (this.ScopeNode.GetType() == typeof(SubnetScopeNode))
                    {
                        SubnetScopeNode node = ((SubnetScopeNode)this.ScopeNode);
                        clients = node.ProfileNode.DhcpOps.SubnetFindAllClients(node.ServerIp, node.DhcpSubnet.Address,
                            clf);
                    }
                    else
                    {
                        ServerScopeNode node = ((ServerScopeNode)this.ScopeNode);
                        clients = node.ProfileNode.DhcpOps.ServerFindAllClients(node.ServerIp, clf);
                    }

                    foreach (DhcpClient client in clients)
                        this.ResultNodes.Add(new LeaseResultNode(client));


                }
                catch (Exception e)
                {
                    SnapInUtil.PopUpError("Search Error", e.Message);
                }

            }

        }

        /// <summary>
        /// Handle when the action is clicked in context menu or action pane
        /// </summary>
        /// <param name="action"></param>
        /// <param name="status"></param>
        protected override void OnAction(Action action, AsyncStatus status)
        {
            if ((string)action.Tag == "ClearSearch")
            {
                // trigger MMC to show the new view
                if (this.ScopeNode.GetType() == typeof(SubnetScopeNode))
                    ((SubnetScopeNode)this.ScopeNode).SwapView(this, "Clear Results", typeof(NormalListView));
                else
                    ((ServerScopeNode)this.ScopeNode).SwapView(this, "Clear Results", typeof(SubnetListView));
            }
        }
    }

    public class NormalListView : MmcListView
    {
        /// <summary>
        /// Constructor
        /// </summary>
        protected override void OnInitialize(AsyncStatus status)
        {
            base.OnInitialize(status);

            this.Columns[0].SetWidth(200);
            
            this.ActionsPaneItems.Add(new Action("Search...", "SearchView", 3, "SearchView"));
        }

        /// <summary>
        /// Handle when the action is clicked in context menu or action pane
        /// </summary>
        /// <param name="action"></param>
        /// <param name="status"></param>
        protected override void OnAction(Action action, AsyncStatus status)
        {
            if ((string)action.Tag == "SearchView")
            {
                // trigger MMC to show the new view
                ((SubnetScopeNode)this.ScopeNode).SwapView(this, "Search...", typeof(SearchListView));
            }
        }
    }
}
