#!/usr/bin/perl 
#
# DHCP Web Services Client
# using SOAP::Lite
#
# Creates a DHCP Subnet and sets DHCP Option 51 (Lease Time) 
#
#
# Copyright (c) 2008 Jason Rupard 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

#use SOAP::Lite +trace => debug;
use SOAP::Lite;

my $dws = SOAP::Lite
  -> uri('http://www.unf.edu/~jrupard/dws/')
  -> proxy('https://jr1/x/DhcpOperations.asmx')
  -> on_action( sub { join '/', 'http://www.unf.edu/~jrupard/dws', $_[1] } );

my $serverIp = '192.168.0.5';
my $networkIp = '192.168.200.64';
my $networkMask = '255.255.255.192';
my $startIp = '192.168.200.65';
my $endIp = '192.168.200.126';

#Server Info
my $serverXML = SOAP::Data->name(
	server => \SOAP::Data->value(SOAP::Data->name(Ip => $serverIp))
    ); 

#Subnet Info
my $subnetXML =
 SOAP::Data->name(network => \SOAP::Data->value(
     SOAP::Data->name(Address => \SOAP::Data->value(SOAP::Data->name(Ip => $networkIp))),
     SOAP::Data->name(Mask => \SOAP::Data->value(SOAP::Data->name(Ip => $networkMask))),
     SOAP::Data->name(Name => 'Made from Perl::Lite'),
     SOAP::Data->name(Comment => 'w00t!'),
     SOAP::Data->name(State => 'Disabled')
     )
 );

#Range Info
my $rangeXML = 
 SOAP::Data->name(ipRange => \SOAP::Data->value(
     SOAP::Data->name(Type => 'DhcpIpRangesDhcpOnly'),
     SOAP::Data->name(StartAddress => \SOAP::Data->value(SOAP::Data->name(Ip => $startIp))),
     SOAP::Data->name(EndAddress => \SOAP::Data->value(SOAP::Data->name(Ip => $endIp)))
     )
 );

#Lease Time
my $optValue51XML = 
 SOAP::Data->name(value => \SOAP::Data->value(
     SOAP::Data->name(OptionId => 51)->type('unsignedInt'),
     SOAP::Data->name(ClassType => 'Dhcp'),
     SOAP::Data->name(OptionData => \SOAP::Data->value(
         SOAP::Data->name(Type => 'DWordType'),
         SOAP::Data->name(Data => \SOAP::Data->value(
             SOAP::Data->name(unsignedInt => 3660))
         )
     ))->attr({ 'xsi:type' => "DhcpOptionDataDword" }),
     SOAP::Data->name(OptionScope => \SOAP::Data->value(
         SOAP::Data->name(type => 'Subnet'),
         SOAP::Data->name(SubnetIp => \SOAP::Data->value(
             SOAP::Data->name(Ip => $networkIp))
         )
     ))
 ));

my $res = $dws->CreateSubnet($serverXML, $subnetXML, $rangeXML);
 die $res->faultstring."\n" if  ($res->fault);
 $res = $dws->SetOptionValue($serverXML, $optValue51XML); 
 die $res->faultstring."\n" if  ($res->fault);


sub SOAP::Transport::HTTP::Client::get_basic_credentials { 
  return "<user>" => "<pass>" ;
}
