#!/usr/bin/perl
#
# DHCP Web Services Client
# using SOAP::Lite
#
# Enumerates Subnets on DHCP Server 
#
#
# Copyright (c) 2008 Jason Rupard 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
# 
#     * Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#

#use SOAP::Lite +trace => debug;
use SOAP::Lite;


my $soap = SOAP::Lite
  -> uri('http://www.unf.edu/~jrupard/dws/')
  -> proxy('https://jr1/x/DhcpOperations.asmx')
  -> on_action( sub { join '/', 'http://www.unf.edu/~jrupard/dws', $_[1] } );

my $serverIp = SOAP::Data->name(
	server => \SOAP::Data->value(SOAP::Data->name(Ip => '<server ip>'))); 

my $res = $soap->EnumSubnets($serverIp);
die $res->faultstring."\n" if not  ($res || $res->fault);

if($res->result->{DhcpSubnet} =~ /ARRAY/) {
 @subnets = @{$res->result->{DhcpSubnet}};
}
else {
 @subnets = ($res->result->{DhcpSubnet});
}

printf "\n%-24s %-12s %-12s %-10s\n", "SubnetName", "Network", "Mask", "State";
printf "%-24s %-12s %-12s %-10s\n", "-"x24, "-"x12, "-"x12, "-"x10;
foreach(@subnets)
{
 printf "%-24s %-12s %-12s %-10s\n", $_->{Name}, $_->{Address}->{Ip}, $_->{Mask}->{Ip}, $_->{State};
}

print "\n";

sub SOAP::Transport::HTTP::Client::get_basic_credentials { 
  return "<user>" => "<pass>";
}
