/* 
 * DHCP Server Management Objects
 *
 *  File: DhcpMacAddress.cs
 *  Namespace: Unf.Dhcp.Smo
 *  
 * 
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
*/
using System;
using System.Collections.Generic;
using System.Text;

using System.Text.RegularExpressions;
using System.Net.NetworkInformation;

namespace Unf.Dhcp.Smo
{
    /// <summary>
    /// Provides a MAC address for the DHCP SMO library
    /// </summary>
    /// <remarks>Accepted formats: 00:01:02:0a:0B:0c, 00-01-02-0a-0B-0c, or 0001020a0B0c</remarks>
    public class DhcpMacAddress
    {

        /// <summary>
        /// Sets Mac to zeros
        /// </summary>
        public DhcpMacAddress() { this.mac = "000000000000"; }
        /// <summary>
        /// Sets Mac to MacString
        /// </summary>
        /// <param name="MacString">String form of Mac address</param>
        /// <remarks>Accepted formats: 00:01:02:0a:0B:0c, 00-01-02-0a-0B-0c, or 0001020a0B0c</remarks>
        public DhcpMacAddress(String MacString) { this.Mac = MacString; }
        /// <summary>
        /// Sets Mac to MacBytes
        /// </summary>
        /// <param name="MacBytes">Array of bytes</param>
        public DhcpMacAddress(Byte[] MacBytes)
        {
            PhysicalAddress InMac = new PhysicalAddress(MacBytes);
            this.mac = InMac.ToString();
        }

        private String mac;
        /// <summary>
        /// Mac address in string form 0001020A0B0C
        /// </summary>
        public String Mac
        {
            get { return this.mac; }
            set
            {
                String InMac = value.ToUpper();

                InMac = InMac.Replace(':', '-');

                this.mac = PhysicalAddress.Parse(InMac).ToString();
            }
        }

        /// <summary>
        /// Mac address in string form
        /// </summary>
        /// <returns>Mac address in string form</returns>
        public override String ToString()
        {
            return Mac;
        }

        /// <summary>
        /// Gets Mac address in byte array form
        /// </summary>
        /// <returns>Byte array form of Mac</returns>
        public Byte[] GetByteArray()
        {
            return PhysicalAddress.Parse(this.mac).GetAddressBytes();
        }
    }
}
