/* 
 * DHCP Server Management Objects
 *
 *  File: DhcpReservation.cs
 *  Namespace: Unf.Dhcp.Smo
 *  
 *
 * Copyright (c) 2008 Jason Rupard 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *     * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
*/
using System;
using System.Collections.Generic;
using System.Text;


namespace Unf.Dhcp.Smo
{
    /// <summary>
    /// Define a DHCP reservation.  A wrapper class of information combined from a client lease and IP reservation object
    /// </summary>
    public class DhcpReservation
    {
        /// <summary>
        /// IP addres of reservation
        /// </summary>
        public DhcpIpAddress ReservedIp;
        /// <summary>
        /// MAC address of reservation
        /// </summary>
        public DhcpMacAddress ReservedMac;
        /// <summary>
        /// Reservation client type (DHCP, BOOTP, or Both)
        /// </summary>
        public DhcpSubnetClientType bAllowedClientTypes;
        /// <summary>
        /// Name of reservation (will be replaced by hostname client get lease)
        /// </summary>
        public String Name;
        /// <summary>
        /// Description of reservation
        /// </summary>
        public String Comment;

        /// <summary>
        /// Dummy constructor
        /// </summary>
        public DhcpReservation() {}
    }
}
